@@ -12,13 +12,40 @@ import ssl
 import h2.connection
 import h2.events
 from urllib3.util import parse_url
-from h2.connection import H2Connection
+from h2.connection import H2Connection,H2Stream
 from h2.events import (
     ResponseReceived, DataReceived, StreamEnded, StreamReset,
     SettingsAcknowledged,
 )
 
 
+def _new_track_content_length(self, length, end_stream):
+    """
+    Update the expected content length in response to data being received.
+    Validates that the appropriate amount of data is sent. Always updates
+    the received data, but only validates the length against the
+    content-length header if one was sent.
+
+    :param length: The length of the body chunk received.
+    :param end_stream: If this is the last body chunk received.
+    """
+    self._actual_content_length += length
+    actual = self._actual_content_length
+    expected = self._expected_content_length
+
+    if expected is not None:
+        if expected < actual:
+            logging.warn(f"InvalidBodyLengthError(expected: {expected}, actual: {actual})")
+            return
+            # raise InvalidBodyLengthError(expected, actual)
+
+        if end_stream and expected != actual:
+            logging.warn(f"InvalidBodyLengthError(expected: {expected}, actual: {actual})")
+            return
+            # raise InvalidBodyLengthError(expected, actual)
+H2Stream._track_content_length = _new_track_content_length
+
+
 class HttpStruct:
     _stream_id: int
     _headers: list = None
